/**************************************
 * Board.cpp
 * Copyright  1996 Steve Sprang
 * All Rights Reserved 
 **************************************/

#include "Board.h"
#include "TGame.h"
#include "Views.h"
#include "BeTetris.h"

bool operator== (rgb_color a, rgb_color b)
{
	return(a.red == b.red && a.green == b.green &&
		a.blue == b.blue);
}

bool operator!= (rgb_color a, rgb_color b)
{
	return(!(a == b));
}

Board::Board(BView *bmapv, TGameView *aview, BBitmap *b, int debris)
{
	int		x,y;
	
	bmapview = bmapv;
	bmap = b;
	arena = aview;
	
	rgb_color debris_color;
	int		  r;
	
	bg_color.red = bg_color.blue = bg_color.green = 0;
	
	for(x = 0; x < WIDTH; x++) {
		for(y = 0; y < HEIGHT - debris; y++) {
			grid[x][y] = bg_color;
		}
		
		for(; y < HEIGHT; y++) {
			r = rand() % 7;
			if(0 <= r && r < 7) {
				debris_color.red = kColors[3*r];
				debris_color.green = kColors[3*r+1];
				debris_color.blue = kColors[3*r+2];
				
				if(rand() % 2) {
					grid[x][y] = debris_color;
				}
				else {
					grid[x][y] = bg_color;
				}
			}
		}
	}
	
	Draw();
}

void Board::Update()
{
	arena->Draw();
}

void Board::Draw()
{
	BRect		square;
	int 		x, y;
	BPoint		tl, tr, bl, br;
	
	bmap->Lock();
	
	bmapview->SetDrawingMode(B_OP_COPY);
	bmapview->SetHighColor(bg_color);
	bmapview->FillRect(bmapview->Bounds());
	
	for(x = 0; x < WIDTH; x++) {
		for(y = 0; y < HEIGHT; y++) {
			if(grid[x][y] != bg_color) {
				bmapview->SetDrawingMode(B_OP_COPY);
				bmapview->SetHighColor(grid[x][y]);
				
				tl.Set(x*DIM, y*DIM);
				tr.Set(x*DIM+DIM-1, y*DIM);
				bl.Set(x*DIM, y*DIM+DIM-1);
				br.Set(x*DIM+DIM-1, y*DIM+DIM-1);
				
				square.SetLeftTop(tl);
				square.SetRightBottom(br);
				bmapview->FillRect(square);
					
				bmapview->SetHighColor(32,32,32);
				
				bmapview->SetDrawingMode(B_OP_SUBTRACT);
				bmapview->StrokeLine(br, bl);
				bmapview->StrokeLine(br, tr);
		
				bmapview->SetDrawingMode(B_OP_ADD);
				bmapview->StrokeLine(tl, bl);
				bmapview->StrokeLine(tl, tr);
			}
		}
	}
	bmapview->Sync();
	bmap->Unlock();
	
	Update();
}

bool Board::EmptyQ(BPoint start, BPoint *points)
{
	BPoint		loc;
	rgb_color	temp;
	int 		i;
	
	for(i = 0; i < SQUARES; i++) {
		loc = start + points[i];
		if(loc.x < 0 || loc.x >= WIDTH || loc.y >= HEIGHT)
			return FALSE;
		else {
			if(loc.y >= 0) {
				temp = grid[(int) loc.x][(int) loc.y];
				if(temp != bg_color) {
					return FALSE;
				}
			}
		}
	}
	return TRUE;
}

void Board::Set(BPoint start, BPoint *points, rgb_color color)
{
	BPoint 		loc;
	int 		i;
	
	for(i = 0; i < SQUARES; i++) {
		loc = start + points[i];
		if(loc.y >= 0) {
			grid[(int) loc.x][(int) loc.y] = color;
		}
	}
}

int Board::RemoveCompletedRows()
{
	int x, y, i = 0, count, blank;
	int	rows[SQUARES];
	
	for(y = (HEIGHT-1); y >= 0; y--) {
		count = blank = 0;
		for(x = 0; x < WIDTH; x++) {
			if(grid[x][y] != bg_color) {
				count++;
			}
			else blank++;
		}
		if(count == WIDTH) {
			rows[i] = y;
			i++;
		}
		if(blank == WIDTH)
			y = -1;
	}
	if(i) {
		FlashRows(rows, i);
		ShiftDown(rows, i);
		Draw();
	}
	
	return i;
}

void Board::ShiftDown(int *rows, int num)
{
	int i,n,v;
	
	for(i = 0; i < num; i++) {
		for(v = rows[i] + i; v > 0; v--) {
			for(n = 0; n < WIDTH; n++) {
				grid[n][v] = grid[n][v-1];
			}
			
		}
		for(int z = 0; z < WIDTH; z++) {
			grid[z][0] = bg_color;
		}
	}
}

void Board::FlashRows(int *rows, int num)
{
	BRect	flashRect;
	int 	i, times, y;
	
	bmap->Lock();
	bmapview->SetDrawingMode(B_OP_BLEND);
	bmapview->SetHighColor(0,0,0);
	bmap->Unlock();
	bmap->Lock();
	for(times = 0; times < 4; times++) {
		
		for(i = 0; i < num; i++) {
			y = rows[i];
			flashRect.Set(0, y*DIM, WIDTH*DIM-1, y*DIM+DIM-1);
			bmapview->FillRect(flashRect);
		}
		bmapview->Sync();
		Update();
		if(times < 3) snooze(60 * 1000);
	}
		bmap->Unlock();
}
