/**************************************
 * TConfigureWindow.cpp
 * Copyright  1996 Steve Sprang
 * All Rights Reserved 
 **************************************/

#include "TConfigureWindow.h"
#include "TWindow.h"
#include "BeTetris.h"

/***************
 * TTextView
 **************/

TTextView::TTextView(BRect bounds, BRect textBound)
	: BTextView(bounds, "", textBound, B_FOLLOW_NONE, B_WILL_DRAW)
{
}

void TTextView::SetText(const char *text)
{
	BTextView::SetText(text);
}

void TTextView::SetText(ulong aChar)
{
	char	key[2];
	
	key[0] = aChar;
	key[1] = NULL;
	
	switch(aChar) {
		case B_TAB:
			SetText("<TAB>");
			break;
		case B_SPACE:
			SetText("<SPACE>");
			break;
		case B_UP_ARROW:
			SetText("<UP>");
			break;
		case B_DOWN_ARROW:
			SetText("<DOWN>");
			break;
		case B_LEFT_ARROW:
			SetText("<LEFT>");
			break;
		case B_RIGHT_ARROW:
			SetText("<RIGHT>");
			break;
		default:
			SetText(key);
			break;
	}
}

void TTextView::KeyDown(ulong aChar)
{
	SetText(aChar);
	value = aChar;
}

/******************
 * TLabelTextView
 *****************/

TLabelTextView::TLabelTextView(BRect bounds, const char *label)
	: BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW)
{
	BRect	leftRect, rightRect, temp;
	BBox	*border;
	
	bounds.OffsetTo(B_ORIGIN);
	leftRect = rightRect = bounds;
	leftRect.right = bounds.Width() * .60;
	rightRect.left = bounds.Width() * .65;
	
	labelField = new BStringView(leftRect, "", label, B_FOLLOW_NONE, B_WILL_DRAW);
	AddChild(labelField);
	
	border = new BBox(rightRect, "");
	AddChild(border);
	
	rightRect.OffsetTo(B_ORIGIN);
	rightRect.InsetBy(1,1);
	editField = new TTextView(rightRect, rightRect);
	border->AddChild(editField);
}

void TLabelTextView::SetText(ulong aChar)
{
	char	key[2];
	
	key[0] = aChar;
	key[1] = NULL;
	
	editField->SetText(aChar);
	editField->value = aChar;
}

void TLabelTextView::AttachedToWindow()
{
	SetViewColor(200,200,200);
	labelField->SetViewColor(200,200,200);
	labelField->SetAlignment(B_ALIGN_RIGHT);
	labelField->SetFontName("Emily");
	editField->SetAlignment(B_ALIGN_CENTER);
}

ulong TLabelTextView::Key()
{
	return editField->value;
}

/*******************
 * TConfigureWindow
 *******************/

void TConfigureWindow::SetKeyFields(ulong l, ulong r, ulong clw, ulong ctcw, ulong d)
{
	left->SetText(l);
	right->SetText(r);
	cw->SetText(clw);
	ccw->SetText(ctcw);
	drop->SetText(d);	
}
	
void TConfigureWindow::WriteKeys()
{	
	BVolume       boot;
	BDirectory	  *root, *dest;
	BFile         *settings;
	ulong		  temp[5];
	
	boot = boot_volume();
	root = new BDirectory();
	dest = new BDirectory();
	settings = new BFile();
	
	boot.GetRootDirectory(root);
	root->GetDirectory("system", dest);
	dest->GetDirectory("settings", dest);
	
	if(dest->GetFile("tetris_settings", settings) < B_NO_ERROR) {
		dest->Create("tetris_settings", settings);	
	}
	
	settings->Open(B_READ_WRITE);
	
	temp[0] = left->Key();
	temp[1] = right->Key();
	temp[2] = cw->Key();
	temp[3] = ccw->Key();
	temp[4] = drop->Key();
	
	settings->Write((ulong *) &temp, sizeof(ulong) * 5);	
	
	settings->Close();
	delete root;
	delete dest;
	delete settings;
}

TConfigureWindow::TConfigureWindow(BRect bounds, TWindow *owner, sem_id dialog_sem)
	: BWindow(bounds, "Configure...", B_MODAL_WINDOW, 
		B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	BRect		rect;
	BBox		*label;
	BButton 	*cancel, *save;
	
	Lock();
	mySem = dialog_sem;
	acquire_sem(mySem);
	
	w = owner;
	
	bounds.OffsetTo(B_ORIGIN);
	bg = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
	AddChild(bg);
	bg->SetViewColor(200,200,200);
	
	rect = bounds;
	rect.OffsetTo(B_ORIGIN);
	rect.InsetBy(10,10);
	rect.bottom -= 30;
	label = new BBox(rect, "");
	label->SetLabel("Key Configurations:");
	bg->AddChild(label);
	label->SetLowColor(200,200,200);
	label->SetViewColor(200,200,200);
	label->SetFontName("Emily");
	
	rect.Set(0,0,130,16);
	rect.OffsetTo(10,20);
	left = new TLabelTextView(rect, "Shift Left:");
	label->AddChild(left);
	
	rect.OffsetBy(140,0);
	right = new TLabelTextView(rect, "Shift Right:");
	label->AddChild(right);
	
	rect.OffsetBy(-140,26);
	ccw = new TLabelTextView(rect, "Rotate CCW:");
	label->AddChild(ccw);
	
	rect.OffsetBy(140,0);
	cw = new TLabelTextView(rect, "Rotate CW:");
	label->AddChild(cw);
	
	rect.OffsetBy(-80,26);
	drop = new TLabelTextView(rect, "Drop:");
	label->AddChild(drop);
	
	rect.Set(0,0,60,20);
	rect.OffsetTo(bounds.Width() - 70, bounds.Height() - 30);
	save = new BButton(rect, "", "Save", new BMessage(mSAVEKEYS));
	bg->AddChild(save);
	save->MakeDefault(TRUE);
	
	rect.Set(0,0,60,20);
	rect.OffsetTo(bounds.Width() - 140, bounds.Height() - 30);
	cancel = new BButton(rect, "", "Cancel", new BMessage(B_QUIT_REQUESTED));
	bg->AddChild(cancel);
	
	Unlock();
}

void TConfigureWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case mSAVEKEYS:
			SaveKeys();
			WriteKeys();
			PostMessage(B_QUIT_REQUESTED);
		break;
	}
}

void TConfigureWindow::SaveKeys()
{
	ulong l, r, c, cc, d;
	BMessage *newKeys = new BMessage(mNEWKEYS);
	
	l = left->Key();
	r = right->Key();
	c = cw->Key();
	cc = ccw->Key();
	d = drop->Key();
	
	newKeys->AddLong("left", l);
	newKeys->AddLong("right", r);
	newKeys->AddLong("cw", c);
	newKeys->AddLong("ccw", cc);
	newKeys->AddLong("drop", d);

	w->PostMessage(newKeys);
}

bool TConfigureWindow::QuitRequested()
{
	release_sem(mySem);
	return TRUE;
}