/**************************************
 * THighScore.cpp
 * Copyright  1996 Steve Sprang
 * All Rights Reserved 
 **************************************/

#include "THighScore.h"
#include "BeTetris.h"

/*************************
 * THighScore
 ************************/

THighScore::THighScore()
{
	ReadScores();
}

void THighScore::OpenScores()
{
	boot = boot_volume();
	root = new BDirectory();
	dest = new BDirectory();
	high_scores = new BFile();
	
	boot.GetRootDirectory(root);
	root->GetDirectory("system", dest);
	dest->GetDirectory("settings", dest);
	strcpy(name, "tetris_data");
}

void THighScore::CloseScores()
{
	delete root;
	delete dest;
	delete high_scores;
}

void THighScore::ReadScores()
{
	OpenScores();
	
	if(dest->GetFile(name, high_scores) < B_NO_ERROR) {
			dest->Create(name, high_scores);
			ClearScores();
	}
	high_scores->Open(B_READ_ONLY);
	high_scores->Read((score_t *) top, sizeof(score_t) * MAX_HIGH_SCORES);
	high_scores->Close();
	
	CloseScores();
}

void THighScore::PromptPlayerName(char *name)
{
	TNameInputWindow	*dialog;
	sem_id 				dialog_sem;
	screen_info			info;
	BRect				dialog_bound;
	
	dialog_sem = create_sem(1, "");
	
	get_screen_info(&info);
	dialog_bound.Set(0, 0, 220, 90);
	dialog_bound.OffsetTo((info.frame.Width() * .5) - (0.5 * dialog_bound.Width()),
		(info.frame.Height() * .5) - (0.5 * dialog_bound.Height()));
	
	dialog = new TNameInputWindow(dialog_bound, name, dialog_sem);
	acquire_sem(dialog_sem);
	delete_sem(dialog_sem);
}

void THighScore::InsertNewScore(score_t newScore)
{
	int 	i, n;
	
	ReadScores();
	
	for(i = 0; i < MAX_HIGH_SCORES; i++) {
		if(newScore.score > top[i].score) {
			PromptPlayerName(newScore.name);
			for(n = MAX_HIGH_SCORES; n > i; n--) {
				top[n] = top [n-1];
			}
			top[i] = newScore;
			i = MAX_HIGH_SCORES;
		}
	}
	WriteScores();
}

void THighScore::WriteScores()
{
	OpenScores();
	
	if(dest->GetFile(name, high_scores) < B_NO_ERROR) {
			dest->Create(name, high_scores);
	}
	else {
		high_scores->Open(B_READ_WRITE);
	}
	high_scores->Write((score_t *) top, sizeof(score_t) * MAX_HIGH_SCORES);
	high_scores->Close();
	
	CloseScores();
}

void THighScore::ClearScores()
{
	int i;
	
	for(i = 0; i < MAX_HIGH_SCORES; i++) {
		strcpy(top[i].name, "");
		top[i].score = 0;
		top[i].level = 0;
		top[i].lines = 0;
	}
}

char* THighScore::PlayerName(int index)
{
	return top[index].name;
}

int THighScore::PlayerScore(int index)
{
	return top[index].score;
}

/*************************
 * TScoreWindow
 ************************/

TScoreWindow::TScoreWindow(BRect bounds, sem_id dialog_sem)
	: BWindow(bounds, "", B_MODAL_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	BRect			rect, textRect, buttonRect;
	BView			*bg;
	BButton 		*clear, *ok;
	BBox 			*label;

	Lock();
	mySem = dialog_sem;
	acquire_sem(mySem);
	
	bounds.OffsetTo(B_ORIGIN);
	bg = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
	AddChild(bg);
	bg->SetViewColor(200,200,200);
	
	rect = bounds;
	rect.InsetBy(10, 10);
	rect.bottom -= 35;
	
	label = new BBox(rect, "");
	bg->AddChild(label);
	label->SetLabel("High Scores:");
	label->SetViewColor(200,200,200);
	label->SetLowColor(200,200,200);
	label->SetHighColor(0,0,255);
	label->SetFontName("Emily");
	
	rect.InsetBy(10, 15);
	textRect = rect;
	textRect.OffsetTo(B_ORIGIN);
	textRect.InsetBy(1,1);
	bounds.OffsetTo(B_ORIGIN);
	
	text = new BTextView(rect, "", textRect, B_FOLLOW_NONE, B_WILL_DRAW);
	bg->AddChild(text);
	text->SetLowColor(200,200,200);
	text->SetViewColor(200,200,200);
	text->SetHighColor(255, 0, 0);
	text->SetFontName("Kate");
	text->MakeEditable(FALSE);
	text->MakeSelectable(FALSE);
	
	buttonRect.Set(0,0,60,20);
	buttonRect.OffsetTo(10, bounds.Height() - 30);
	clear = new BButton(buttonRect, "", "Clear", new BMessage(mCLEARSCORES));
	bg->AddChild(clear);
	
	buttonRect.OffsetTo(bounds.Width() - 70, bounds.Height() - 30);
	ok = new BButton(buttonRect, "", "OK", new BMessage(B_QUIT_REQUESTED));
	bg->AddChild(ok);
	ok->MakeDefault(TRUE);
	
	Show();
	Unlock();
}

bool TScoreWindow::QuitRequested()
{
	release_sem(mySem);
	return TRUE;
}

void TScoreWindow::MessageReceived(BMessage *msg)
{
	BAlert		*alrt;
	
	switch(msg->what) {
		case mCLEARSCORES:
			beep();
			alrt = new BAlert("", "Really clear the high scores?", "Cancel", "OK",
				NULL, B_WIDTH_AS_USUAL, B_WARNING_ALERT);
			alrt->SetShortcut(0, B_ESCAPE);
			alrt->SetShortcut(1, B_ENTER);
			if(alrt->Go()) {
				scores->ClearScores();
				scores->WriteScores();
				SetScores(scores);
			}
		break;
	}
}

void TScoreWindow::SetScores(THighScore *new_scores)
{
	char		line[80], body[1200];
	
	body[0] = NULL;
	
	Lock();
	scores = new_scores;
	scores->ReadScores();
	
	
	for(int n = 0; n < MAX_HIGH_SCORES; n++) {
		sprintf(line, "%3d. %-30s%12d\n", n+1, scores->PlayerName(n), 
			scores->PlayerScore(n));
		strcat(body, line);
	}
	text->SetText(body);
	Unlock();
}

/*************************
 * TNameInputWindow
 ************************/

TNameInputWindow::TNameInputWindow(BRect bound, char *player_name, sem_id dialog_sem)
	: BWindow(bound, "", B_MODAL_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	BRect	rect, textRect;
	BView	*bg;
	BButton	*ok;
	
	acquire_sem(dialog_sem);
	Lock();
	
	this->name = player_name;
	mySem = dialog_sem;
	
	bound.OffsetTo(B_ORIGIN);
	bg = new BView(bound, "", B_FOLLOW_NONE, B_WILL_DRAW);
	AddChild(bg);
	bg->SetViewColor(200, 200, 200);
	
	rect.Set(0, 0, 200, 16);
	rect.OffsetBy(10,10);
	prompt = new BStringView(rect, "", "High Score! Enter your name:");
	bg->AddChild(prompt);
	prompt->SetFontName("Emily");
	prompt->SetViewColor(200, 200, 200);
	
	rect.OffsetBy(0, 24);
	border = new BBox(rect, "");
	bg->AddChild(border);
	
	rect.OffsetTo(B_ORIGIN);
	rect.InsetBy(1,1);
	textRect = rect;
	textRect.OffsetTo(B_ORIGIN);
	textRect.InsetBy(1,1);
	
	input = new BTextView(rect, "", textRect, B_FOLLOW_NONE, B_WILL_DRAW);
	border->AddChild(input);
	
	rect.Set(0, 0, 60, 20);
	rect.OffsetTo(bound.Width() - 70, bound.Height() - 30);
	ok = new BButton(rect, "", "OK", new BMessage(B_QUIT_REQUESTED));
	bg->AddChild(ok);
	ok->MakeDefault(TRUE);
	
	input->MakeFocus();
	
	Show();
	Unlock();
}

bool TNameInputWindow::QuitRequested()
{
	strcpy(name, input->Text());
	
	release_sem(mySem);
	return TRUE;
}
