/**************************************
 * TWindow.cpp
 * Copyright  1996 Steve Sprang
 * All Rights Reserved 
 **************************************/

#include "TWindow.h"
#include "TGame.h"
#include "Views.h"
#include "Board.h"
#include "BeTetris.h"
#include "THighScore.h"
#include "TConfigureWindow.h"

bool TWindow::QuitRequested()
{
	if(theGame) theGame->PostMessage(mQUIT);
	be_app->PostMessage(B_QUIT_REQUESTED);
	
	return(TRUE);
}

TWindow::~TWindow()
{
	delete nextoff;
	delete arenaoff;
}


TWindow::TWindow(BRect bounds, const char *title)
	: BWindow(bounds, title, B_TITLED_WINDOW, 
		B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	BBox			*nextBorder, *arenaBox;
	BRect			arenaBound, borderBound, nextBound;
	BRect			stringBound;
	BView			*bg;
	BStringView		*nextLabel;
	
	Lock();
	bounds.OffsetTo(B_ORIGIN);
	
	// background
	bg = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
	AddChild(bg);
	
	// menus
	bounds.bottom = 16;
	SetUpMenus(bounds);
	bg->AddChild(menuBar);
	bg->SetViewColor(200,200,200);
	
	// next label
	stringBound.Set(0,0,60, 14);
	stringBound.OffsetTo(DIM*WIDTH+26, 24);
	nextLabel = new BStringView(stringBound, "", "Next:");
	bg->AddChild(nextLabel);
	nextLabel->SetFontName("Emily");
	nextLabel->SetViewColor(200,200,200);
	
	// next 
	borderBound.Set(0,0,DIM*5,DIM*5);
	nextBound = borderBound;
	borderBound.OffsetTo(DIM*WIDTH+26, 43);
	nextBorder = new BBox(borderBound, "");
	bg->AddChild(nextBorder);
	nextBorder->SetHighColor(100,100,100);
	nextBorder->SetViewColor(0,0,0);
	
	nextBound.InsetBy(1,1);
	nextBound.OffsetTo(B_ORIGIN);
	
	nextoff = new BBitmap(nextBound, B_COLOR_8_BIT, TRUE);
	nextview = new BView(nextBound, "", B_FOLLOW_ALL, B_WILL_DRAW);
	nextoff->AddChild(nextview);
	nextoff->Lock();
	nextview->SetHighColor(0,0,0);
	nextview->FillRect(nextBound);
	nextview->Sync();
	nextoff->Unlock();
	
	nextBound.OffsetTo(BPoint(1,1));
	next = new TBlitView(nextBound, nextoff);
	nextBorder->AddChild(next);
	next->SetViewColor(0,0,0);
	
	// string views
	stringBound.Set(0,0,DIM*5, 30);
	stringBound.OffsetTo(DIM*WIDTH+26, DIM*5+43+12);
	score = new TStringView(stringBound, "Score:");
	bg->AddChild(score);
	
	stringBound.OffsetBy(0, 40);
	lines = new TStringView(stringBound, "Lines:");
	bg->AddChild(lines);
	
	stringBound.OffsetBy(0, 40);
	level = new TStringView(stringBound, "Level:");
	bg->AddChild(level);
		
	// arena
	arenaBound.Set(0, 0, DIM*WIDTH+1, DIM*HEIGHT+1);
	arenaBound.OffsetBy(10, 26);
	arenaBox = new BBox(arenaBound, "");
	bg->AddChild(arenaBox);
	arenaBox->SetHighColor(100,100,100);
	arenaBox->SetViewColor(0,0,0);
	
	arenaBound.InsetBy(1,1);
	arenaBound.OffsetTo(B_ORIGIN);
		
	arenaoff = new BBitmap(arenaBound, B_COLOR_8_BIT, TRUE);
	arenaview = new BView(arenaBound, "", B_FOLLOW_ALL, B_WILL_DRAW);
	arenaoff->AddChild(arenaview);
	arenaoff->Lock();
	arenaview->SetHighColor(0,0,0);
	arenaview->FillRect(arenaview->Bounds());
	arenaview->Sync();
	arenaoff->Unlock();
	
	arenaBound.OffsetTo(BPoint(1.0,1.0));
	arena = new TGameView(arenaBound, arenaoff, arenaview);
	arenaBox->AddChild(arena);
	arena->SetViewColor(0,0,0);
	
	theGame = NULL;
	arena->MakeFocus();
	
	hs = new THighScore();
	left = 'j';
	right = 'l';
	drop = ' ';
	cw = 'o';
	ccw = 'u';
	
	start_level = 1;
	debris_rows = 0;
	
	ReadKeys();
	Unlock();
}

window_pointers_t TWindow::GetViewPointers()
{
	window_pointers_t		v;
	
	v.arena = arena;
	v.next = next;
	v.arenaoff = arenaoff;
	v.nextoff = nextoff;
	v.arenaview = arenaview;
	v.nextview = nextview;
	v.score = score;
	v.lines = lines;
	v.level = level;
	v.hs = hs;
	
	return v;
}

void TWindow::ReadKeys()
{	
	BVolume       boot;
	BDirectory	  *root, *dest;
	BFile         *settings;
	
	boot = boot_volume();
	root = new BDirectory();
	dest = new BDirectory();
	settings = new BFile();
	
	boot.GetRootDirectory(root);
	root->GetDirectory("system", dest);
	dest->GetDirectory("settings", dest);
	
	if(dest->GetFile("tetris_settings", settings) < B_NO_ERROR) {
		dest->Create("tetris_settings", settings);
		settings->Open(B_READ_WRITE);
		settings->Write((ulong *) &left, sizeof(ulong));
		settings->Write((ulong *) &right, sizeof(ulong));
		settings->Write((ulong *) &cw, sizeof(ulong));
		settings->Write((ulong *) &ccw, sizeof(ulong));
		settings->Write((ulong *) &drop, sizeof(ulong));	
	}
	else {
		settings->Open(B_READ_ONLY);
		settings->Read((ulong *) &left, sizeof(ulong));
		settings->Read((ulong *) &right, sizeof(ulong));
		settings->Read((ulong *) &cw, sizeof(ulong));
		settings->Read((ulong *) &ccw, sizeof(ulong));
		settings->Read((ulong *) &drop, sizeof(ulong));	
	}
	settings->Close();
	delete root;
	delete dest;
	delete settings;
}

void TWindow::SetUpMenus(BRect bounds)
{
	BMenu		*play, *options, *level_menu, *debris_menu;
	BMenuItem	*newItem;
	char		numString[8];
	BMessage	*msg;
	
	// create the menu bar
	menuBar = new BMenuBar(bounds, "menubar", B_FOLLOW_ALL, B_ITEMS_IN_ROW,
		FALSE);
		
	// create the "Play" and "Options" menus
	play = new BMenu("Play", B_ITEMS_IN_COLUMN);
	options = new BMenu("Options", B_ITEMS_IN_COLUMN);
	level_menu = new BMenu("Start at Level", B_ITEMS_IN_COLUMN);
	debris_menu = new BMenu("Rows of Debris", B_ITEMS_IN_COLUMN);
	
	level_menu->SetRadioMode(TRUE);
	debris_menu->SetRadioMode(TRUE);
	
	// add items to "Start at Level" submenu
	for(int i = 1; i <= 10; i++) {
		sprintf(numString, "%d", i);
		msg = new BMessage(mSTARTLEVEL);
		msg->AddLong("level", i);
		newItem = new BMenuItem(numString, msg, NULL, NULL);
		level_menu->AddItem(newItem);
		if(i == 1) {
			newItem->SetMarked(TRUE);
		}
	}
	
	// add items to "Rows of Debris" submenu
	for(int i = 0; i < 15; i++) {
		sprintf(numString, "%d", i);
		msg = new BMessage(mDEBRISROWS);
		msg->AddLong("debris", i);
		newItem = new BMenuItem(numString, msg, NULL, NULL);
		debris_menu->AddItem(newItem);
		if(i == 0) {
			newItem->SetMarked(TRUE);
		}
	}
	
	// add items to "Play" menu
	newItem = new BMenuItem("New Game", new BMessage(mBEGIN), 'G', NULL);
	play->AddItem(newItem);
	play->AddSeparatorItem();
	newItem = new BMenuItem("Toggle Pause", new BMessage(mPAUSE), 'P', NULL);
	play->AddItem(newItem);
	newItem = new BMenuItem("Abort", new BMessage(mABORT), 'A', NULL);
	play->AddItem(newItem);
	
	// add items to "Options" menu
	newItem = new BMenuItem("Configure Keys", new BMessage(mCONFIGURE), 'C', NULL);
	options->AddItem(newItem);
	options->AddSeparatorItem();
	newItem = new BMenuItem("Show High Scores...", new BMessage(mHIGH), 'H', NULL);
	options->AddItem(newItem);
	
	options->AddSeparatorItem();
	newItem = new BMenuItem(level_menu);
	options->AddItem(newItem);
	newItem = new BMenuItem(debris_menu);
	options->AddItem(newItem);

	// insert the two menus into the menu bar
	menuBar->AddItem(play);
	menuBar->AddItem(options);
}

void TWindow::SetGameToNull()
{
	theGame = NULL;
}

void TWindow::MessageReceived(BMessage *message)
{
	Lock();
	switch(message->what) {
		case mBEGIN:
			if(!theGame) {
				theGame = new TGame(this, start_level, debris_rows);
				theGame->SetKeys(left, right, cw, ccw, drop);
				arena->SetGame(theGame);
				theGame->Run();
			}
			else {
				beep();
			}
			break;
		
		case mPAUSE:
			if(theGame) theGame->TogglePause();
			break;

		case mABORT:
			if(theGame) theGame->GameOver();
			break;
			
		case mHIGH:
			ShowHighScores();
			break;
			
		case mCONFIGURE:
			OpenConfigureWindow();
			break;
			
		case mNEWKEYS:
			SetKeys(message);
			break;
			
		case mSTARTLEVEL:
			start_level = message->FindLong("level");
			break;
			
		case mDEBRISROWS:
			debris_rows = message->FindLong("debris");
			break;
	}
	Unlock();
}

void TWindow::SetKeys(BMessage *message)
{
	left = message->FindLong("left");
	right = message->FindLong("right");
	cw = message->FindLong("cw");
	ccw = message->FindLong("ccw");
	drop = message->FindLong("drop");
	
	if(theGame) {
		theGame->SetKeys(left, right, cw, ccw, drop);
	}
}


void TWindow::OpenConfigureWindow()
{
	TConfigureWindow	*config;
	bool 				state;
	BRect				bounds;
	screen_info			info;
	sem_id				dialog_sem;
	
	get_screen_info(&info);
	dialog_sem = create_sem(1, "");
	
	if(theGame) {
		state = theGame->PausedQ();
		if(!state) {
			theGame->TogglePause();
		}
	}		
		
	bounds.Set(0,0,330,150);
	bounds.OffsetTo((info.frame.Width() * .5) - (0.5 * bounds.Width()),
		(info.frame.Height() * .5) - (0.5 * bounds.Height()));
		
	config = new TConfigureWindow(bounds, this, dialog_sem);
	config->SetKeyFields(left, right, cw, ccw, drop);
	config->Show();
	
	acquire_sem(dialog_sem);
	delete_sem(dialog_sem);
	
	if(theGame && !state) {
		theGame->TogglePause();
	}
}

void TWindow::ShowHighScores()
{
	BRect			bounds;
	screen_info		info;
	bool			state;
	sem_id			dialog_sem;
	TScoreWindow 	*sWin;
	
	get_screen_info(&info);
	dialog_sem = create_sem(1, "");
	
	if(theGame) {
		state = theGame->PausedQ();
		if(!state) {
			theGame->TogglePause();
		}
	}	
	
	get_screen_info(&info);
	
	bounds.Set(0,0, 380, 250);
	bounds.OffsetTo((info.frame.Width() * .5) - (0.5 * bounds.Width()),
		(info.frame.Height() * .5) - (0.5 * bounds.Height()));
	sWin = new TScoreWindow(bounds, dialog_sem);
	sWin->SetScores(hs);
	
	acquire_sem(dialog_sem);
	delete_sem(dialog_sem);

	if(theGame && !state) {
		theGame->TogglePause();
	}
}

