/**************************************
 * Tetris.cpp
 * Copyright  1996 Steve Sprang
 * All Rights Reserved 
 **************************************/

#include "Tetris.h"


Tetris::Tetris(Board *aboard, BView *bmapview, TGame *agame) 
{
	int piece;
	
	pivotLoc.Set(5.0, -1.0);
	
	myGame = agame;
	myBoard = aboard;
	offscreen = bmapview;
	canRotate = TRUE;
	
	piece = rand() % 7;
	if(!piece)  // the 'block' doesn't rotate and it acts funny if you try
		canRotate = FALSE;
		
	for(int i = 0; i < 8; i+=2) {
		layout[i / 2].Set(kPieceDef[piece][i],
			kPieceDef[piece][i+1]);
	}

	color.red = kColors[piece*3];
	color.green = kColors[piece*3+1];
	color.blue = kColors[piece*3+2];
}

void Tetris::Rotate(int dir)
{
	BPoint		query[SQUARES];
	int 		i;
	
	if(canRotate) {
		for(i = 0; i < SQUARES; i++) {
			if(dir == CW) {
				query[i].x = - layout[i].y;
				query[i].y = layout[i].x;
			}
			else {	
				query[i].x = layout[i].y;
				query[i].y = -layout[i].x;
			}
		}
		
		Erase();
		if(myBoard->EmptyQ(pivotLoc, query)) {
			for(i = 0; i < SQUARES; i++) {
				layout[i] = query[i];
			}
		}
		Draw();
	}
}


int Tetris::ShiftDown()
{
	BPoint		nextLoc = pivotLoc;
	int			i, status;
	
	nextLoc.y++;
	
	Erase();
	if(myBoard->EmptyQ(nextLoc, layout)) {
		pivotLoc = nextLoc;
		status = 0;
	}
	else {
		Set();
		status = 1;
		for(i = 0; i < SQUARES; i++) {
			if(pivotLoc.y + layout[i].y < 0) {
				status++;
			}
		}
	}
	Draw();
	return status;
}

void Tetris::Shift(int dir)
{
	BPoint	nextLoc = pivotLoc;
	
	if(dir == LEFT)
		nextLoc.x--;
	else 
		nextLoc.x++;
	
	Erase();
	if(myBoard->EmptyQ(nextLoc, layout)) {
		pivotLoc = nextLoc;
	}
	Draw();
}

bool Tetris::Drop()
{
	BPoint	nextLoc = pivotLoc;
	bool	moved = FALSE, visible = FALSE;
	int 	i;
	
	for(i = 0; i < SQUARES; i++) {
		if(pivotLoc.y + layout[i].y >= 0) {
			visible = TRUE;
		}
	}
	
	if(visible) {
		Erase();
		while(myBoard->EmptyQ(nextLoc, layout)) {
			nextLoc.y++;
			moved = TRUE;
		}
		if(moved) nextLoc.y--;
		
		pivotLoc = nextLoc;
		Set();
		Draw();
	}
	return moved;
}

void Tetris::Set()
{
	myBoard->Set(pivotLoc, layout, color);
}

void Tetris::Erase()
{
	BRect		square;
	int 		i;
	BPoint		tl, tr, bl, br;
	BPoint		temp;
	
	offscreen->Window()->Lock();
	
	for(i = 0; i < SQUARES; i++) {
		temp = pivotLoc + layout[i];
		
		offscreen->SetDrawingMode(B_OP_COPY);
		offscreen->SetHighColor(0,0,0);
				
		tl.Set(temp.x*DIM, temp.y*DIM);
		tr.Set(temp.x*DIM+DIM-1, temp.y*DIM);
		bl.Set(temp.x*DIM, temp.y*DIM+DIM-1);
		br.Set(temp.x*DIM+DIM-1, temp.y*DIM+DIM-1);
		
		square.SetLeftTop(tl);
		square.SetRightBottom(br);
		offscreen->FillRect(square);
	}
	offscreen->Sync();
	offscreen->Window()->Unlock();
}

void Tetris::DrawSquare(BView *which)
{
	BRect	square;
	
	which->SetDrawingMode(B_OP_COPY);
	which->SetHighColor(color);
	
	square.SetLeftTop(tl);
	square.SetRightBottom(br);
	which->FillRect(square);
	
	which->SetHighColor(32,32,32);
	
	which->SetDrawingMode(B_OP_SUBTRACT);
	which->StrokeLine(br, bl);
	which->StrokeLine(br, tr);

	which->SetDrawingMode(B_OP_ADD);
	which->StrokeLine(tl, bl);
	which->StrokeLine(tl, tr);
}

void Tetris::Draw() 
{
	int 		i;
	BPoint		temp;
	
	offscreen->Window()->Lock();
	for(i = 0; i < SQUARES; i++) {
		temp = pivotLoc + layout[i];
				
		tl.Set(temp.x*DIM, temp.y*DIM);
		tr.Set(temp.x*DIM+DIM-1, temp.y*DIM);
		bl.Set(temp.x*DIM, temp.y*DIM+DIM-1);
		br.Set(temp.x*DIM+DIM-1, temp.y*DIM+DIM-1);
		
		DrawSquare(offscreen);
	}
	offscreen->Sync();
	offscreen->Window()->Unlock();
	myBoard->Update();
}

void Tetris::DrawNext(BView *nextv, TBlitView *updt)
{
	BRect 		bounds, un;
	BPoint 		temp;
	int 		i, h_offset, v_offset, low_h = SQUARES, low_v = SQUARES;
	
	nextv->Window()->Lock();
	bounds = nextv->Bounds();
	nextv->SetDrawingMode(B_OP_COPY);
	nextv->SetHighColor(0,0,0);
	nextv->FillRect(bounds);
	
	un.Set(0,0,0,0);
	
	for(i = 0; i < SQUARES; i++) {
		temp = layout[i];
		
		if(temp.x < low_h) low_h = temp.x;
		if(temp.y < low_v) low_v = temp.y;
						
		tl.Set(temp.x * DIM, temp.y * DIM);
		br.Set(temp.x*DIM+DIM-1, temp.y*DIM+DIM-1);
		
		un = un | BRect(tl, br);
	}

	h_offset = (int) ((bounds.Width() - un.Width()) / 2);
	v_offset = (int) ((bounds.Height() - un.Height()) / 2);
	
	h_offset -= low_h * DIM;
	v_offset -= low_v * DIM;
	
	for(i = 0; i < SQUARES; i++) {
		temp = layout[i];
		
		tl.Set(h_offset + temp.x*DIM, v_offset + temp.y*DIM);
		tr.Set(h_offset + temp.x*DIM+DIM-1, v_offset + temp.y*DIM);
		bl.Set(h_offset + temp.x*DIM, v_offset + temp.y*DIM+DIM-1);
		br.Set(h_offset + temp.x*DIM+DIM-1, v_offset + temp.y*DIM+DIM-1);
		
		DrawSquare(nextv);
	}
	nextv->Sync();
	nextv->Window()->Unlock();
	updt->Draw(bounds);
}

	
