/***************************************
 *  Views.cpp
 *	Copyright  1996 Steve Sprang
 *  All Rights Reserved.
 ***************************************/

#include "Views.h"
#include "BeTetris.h"

/****************
 * TBlitView
 ***************/

TBlitView::TBlitView(BRect bound, BBitmap *b)
	: BView(bound, "", B_FOLLOW_NONE, B_WILL_DRAW)
{
	offscreen = b;
}

void TBlitView::Draw(BRect update)
{
	Draw();
}

void TBlitView::Draw()
{
	Window()->Lock();
	DrawBitmap(offscreen);
	Window()->Unlock();
}

/****************
 * TGameView
 ***************/

TGameView::TGameView(BRect bound, BBitmap *o, BView *v)
	: TBlitView(bound, o)
{
	offview = v;
	game = NULL;
}

void TGameView::SetGame(TGame *g)
{
	game = g;
}

void TGameView::KeyDown(ulong aChar)
{
	BMessage	*msg = new BMessage(mKEYDOWN);
	
	msg->AddLong("key", aChar);
	
	if(game) {
		game->PostMessage(msg);
	}
}

/****************
 * TStringView
 ***************/

TStringView::TStringView(BRect bound, const char *lbl)
 : BView(bound, "", B_FOLLOW_NONE, B_WILL_DRAW)
{
	BRect	labelFrame, valueFrame;
	
	bound.OffsetTo(B_ORIGIN);
	labelFrame = valueFrame = bound;
	labelFrame.bottom -= bound.Height() * .5 - 1;
	valueFrame.top += bound.Height() * .5 + 1;
	
	label = new BStringView(labelFrame, "", lbl);
	AddChild(label);
	border = new BBox(valueFrame, "");
	AddChild(border);
	valueFrame.OffsetTo(B_ORIGIN);
	valueFrame.InsetBy(1,1);
	string = new BStringView(valueFrame, "", "");
	border->AddChild(string);
}

void TStringView::AttachedToWindow()
{
	label->SetViewColor(200,200,200);
	label->SetLowColor(200,200,200);
	label->SetFontName("Emily");
	
	string->SetAlignment(B_ALIGN_RIGHT);
}

void TStringView::SetValue(int val)
{
	char	text[32], *current;
	int 	old;
	
	current = string->Text();
	sscanf(current, "%d", &old);
	
	if(val != old) {
		sprintf(text, "%d", val);
		string->SetText(text);
	}
}